@@ -112,6 +112,11 @@ static void       xfce_appfinder_window_icon_theme_changed            (XfceAppfi
 static void       xfce_appfinder_window_launch_clicked                (XfceAppfinderWindow         *window);
 static void       xfce_appfinder_window_execute                       (XfceAppfinderWindow         *window,
                                                                        gboolean                     close_on_succeed);
+static gint       xfce_appfinder_window_sort_items                    (GtkTreeModel                *model,
+                                                                       GtkTreeIter                 *a,
+                                                                       GtkTreeIter                 *b,
+                                                                       gpointer                     data);
+static gboolean   xfce_appfinder_should_sort_icon_view                (void);
 
 struct _XfceAppfinderWindowClass
 {
@@ -124,6 +129,9 @@ struct _XfceAppfinderWindow
 
   XfceAppfinderModel         *model;
 
+  GtkTreeModel               *sort_model;
+  GtkTreeModel               *filter_model;
+
   XfceAppfinderCategoryModel *category_model;
 
   XfceAppfinderActions       *actions;
@@ -345,6 +353,7 @@ xfce_appfinder_window_init (XfceAppfinderWindow *window)
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "clicked",
       G_CALLBACK (xfce_appfinder_window_preferences), window);
+  gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
   gtk_button_set_always_show_image(GTK_BUTTON(button), TRUE);
 
   image = gtk_image_new_from_icon_name (XFCE_APPFINDER_STOCK_PREFERENCES, GTK_ICON_SIZE_BUTTON);
@@ -418,6 +427,8 @@ xfce_appfinder_window_finalize (GObject *object)
 
   g_object_unref (G_OBJECT (window->model));
   g_object_unref (G_OBJECT (window->category_model));
+  g_object_unref (G_OBJECT (window->filter_model));
+  g_object_unref (G_OBJECT (window->sort_model));
   g_object_unref (G_OBJECT (window->completion));
   g_object_unref (G_OBJECT (window->icon_find));
 
@@ -664,7 +675,6 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer   *renderer;
-  GtkTreeModel      *filter_model;
   GtkTreeSelection  *selection;
   GtkWidget         *view;
   gboolean           icon_view;
@@ -677,13 +687,20 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       gtk_widget_destroy (window->view);
     }
 
-  filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (window->model), NULL);
-  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter_model), xfce_appfinder_window_item_visible, window, NULL);
+  window->filter_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (window->model), NULL);
+  gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (window->filter_model), xfce_appfinder_window_item_visible, window, NULL);
+
+  window->sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (window->filter_model));
+  gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->sort_model), xfce_appfinder_window_sort_items, window->entry, NULL);
 
   if (icon_view)
     {
-      window->view = view = gtk_icon_view_new_with_model (filter_model);
-      gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (view), GTK_SELECTION_BROWSE);
+      window->view = view = gtk_icon_view_new_with_model (
+        xfce_appfinder_should_sort_icon_view () ?
+          window->sort_model :
+          window->filter_model);
+
+      gtk_icon_view_set_selection_mode (GTK_ICON_VIEW (view), GTK_SELECTION_SINGLE);
       gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_ICON);
       gtk_icon_view_set_text_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TITLE);
       gtk_icon_view_set_tooltip_column (GTK_ICON_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TOOLTIP);
@@ -696,7 +713,7 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
     }
   else
     {
-      window->view = view = gtk_tree_view_new_with_model (filter_model);
+      window->view = view = gtk_tree_view_new_with_model (window->sort_model);
       gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
       gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), FALSE);
       gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (view), XFCE_APPFINDER_MODEL_COLUMN_TOOLTIP);
@@ -708,7 +725,7 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
            G_CALLBACK (xfce_appfinder_window_treeview_key_press_event), window);
 
       selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-      gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+      gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
       g_signal_connect_swapped (G_OBJECT (selection), "changed",
           G_CALLBACK (xfce_appfinder_window_item_changed), window);
 
@@ -741,8 +758,6 @@ xfce_appfinder_window_view (XfceAppfinderWindow *window)
       G_CALLBACK (xfce_appfinder_window_view_button_press_event), window);
   gtk_container_add (GTK_CONTAINER (window->viewscroll), view);
   gtk_widget_show (view);
-
-  g_object_unref (G_OBJECT (filter_model));
 }
 
 
@@ -788,6 +803,23 @@ xfce_appfinder_window_view_get_selected (XfceAppfinderWindow  *window,
 
 
 
+static gboolean
+xfce_appfinder_window_view_get_selected_path (XfceAppfinderWindow  *window,
+                                              GtkTreePath         **path)
+{
+  GtkTreeModel *model;
+  GtkTreeIter   iter;
+
+  if (!xfce_appfinder_window_view_get_selected (window, &model, &iter))
+    return FALSE;
+
+  *path = gtk_tree_model_get_path (model, &iter);
+
+  return (*path != NULL);
+}
+
+
+
 static void
 xfce_appfinder_window_popup_menu_toggle_bookmark (GtkWidget           *mi,
                                                   XfceAppfinderWindow *window)
@@ -796,7 +828,6 @@ xfce_appfinder_window_popup_menu_toggle_bookmark (GtkWidget           *mi,
   GFile        *gfile;
   gchar        *desktop_id;
   GtkWidget    *menu = gtk_widget_get_parent (mi);
-  GtkTreeModel *filter;
   GtkTreeModel *model;
   GError       *error = NULL;
 
@@ -808,8 +839,7 @@ xfce_appfinder_window_popup_menu_toggle_bookmark (GtkWidget           *mi,
       g_object_unref (G_OBJECT (gfile));
 
       /* toggle bookmarks */
-      filter = g_object_get_data (G_OBJECT (menu), "model");
-      model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter));
+      model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (window->filter_model));
       xfce_appfinder_model_bookmark_toggle (XFCE_APPFINDER_MODEL (model), desktop_id, &error);
 
       g_free (desktop_id);
@@ -1083,7 +1113,14 @@ xfce_appfinder_window_popup_menu (GtkWidget           *view,
           G_CALLBACK (xfce_appfinder_window_popup_menu_hide), window);
       gtk_widget_show (mi);
 
+#if GTK_CHECK_VERSION (3, 22, 0)
       gtk_menu_popup_at_pointer (GTK_MENU (menu), NULL);
+#else
+      gtk_menu_popup (GTK_MENU (menu),
+                      NULL, NULL, NULL, NULL, 3,
+                      gtk_get_current_event_time ());
+#endif
+
       return TRUE;
     }
 
@@ -1152,8 +1189,8 @@ xfce_appfinder_window_entry_changed_idle (gpointer data)
   const gchar         *text;
   GdkPixbuf           *pixbuf;
   gchar               *normalized;
-  GtkTreeModel        *model;
-
+  GtkTreePath         *path;
+  GtkTreeSelection    *selection;
 
   text = gtk_entry_get_text (GTK_ENTRY (window->entry));
 
@@ -1173,11 +1210,47 @@ xfce_appfinder_window_entry_changed_idle (gpointer data)
         }
 
       APPFINDER_DEBUG ("refilter entry");
+
+      gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (window->filter_model));
+      APPFINDER_DEBUG ("FILTER TEXT: %s\n", window->filter_text);
+
       if (GTK_IS_TREE_VIEW (window->view))
-        model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
+        {
+          gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (window->view), 0, 0);
+
+          if (window->filter_text == NULL)
+            {
+              /* if filter is empty, clear selection */
+              selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->view));
+              gtk_tree_selection_unselect_all (selection);
+
+              /* reset cursor */
+              path = gtk_tree_path_new_first ();
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
+              gtk_tree_path_free (path);
+            }
+          else if (gtk_tree_view_get_visible_range (GTK_TREE_VIEW (window->view), &path, NULL))
+            {
+              /* otherwise select the first match */
+              gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
+              gtk_tree_path_free (path);
+            }
+        }
       else
-        model = gtk_icon_view_get_model (GTK_ICON_VIEW (window->view));
-      gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (model));
+        {
+          if (window->filter_text == NULL)
+            {
+              /* if filter is empty, clear selection */
+              gtk_icon_view_unselect_all (GTK_ICON_VIEW (window->view));
+            }
+          else if (gtk_icon_view_get_visible_range (GTK_ICON_VIEW (window->view), &path, NULL))
+            {
+              /* otherwise select the first match */
+              gtk_icon_view_select_path (GTK_ICON_VIEW (window->view), path);
+              gtk_icon_view_set_cursor (GTK_ICON_VIEW (window->view), path, NULL, FALSE);
+              gtk_tree_path_free (path);
+            }
+        }
     }
   else
     {
@@ -1220,33 +1293,25 @@ static void
 xfce_appfinder_window_entry_activate (GtkEditable         *entry,
                                       XfceAppfinderWindow *window)
 {
-  GtkTreePath *path;
-  gboolean     cursor_set = FALSE;
+  GList            *selected_items;
+  GtkTreeSelection *selection;
+  gboolean          has_selection = FALSE;
 
   if (gtk_widget_get_visible (window->paned))
     {
       if (GTK_IS_TREE_VIEW (window->view))
         {
-          if (gtk_tree_view_get_visible_range (GTK_TREE_VIEW (window->view), &path, NULL))
-            {
-              gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
-              gtk_tree_path_free (path);
-
-              cursor_set = TRUE;
-            }
+          selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->view));
+          has_selection = gtk_tree_selection_count_selected_rows (selection) > 0;
         }
-      else if (gtk_icon_view_get_visible_range (GTK_ICON_VIEW (window->view), &path, NULL))
-        {
-          gtk_icon_view_select_path (GTK_ICON_VIEW (window->view), path);
-          gtk_icon_view_set_cursor (GTK_ICON_VIEW (window->view), path, NULL, FALSE);
-          gtk_tree_path_free (path);
-
-          cursor_set = TRUE;
-        }
-
-      if (cursor_set)
-        gtk_widget_grab_focus (window->view);
       else
+        {
+          selected_items = gtk_icon_view_get_selected_items (GTK_ICON_VIEW (window->view));
+          has_selection = (selected_items != NULL);
+          g_list_free_full (selected_items, (GDestroyNotify) gtk_tree_path_free);
+        }
+
+      if (has_selection)
         xfce_appfinder_window_execute (window, TRUE);
     }
   else if (gtk_widget_get_sensitive (window->button_launch))
@@ -1279,17 +1344,26 @@ xfce_appfinder_window_entry_key_press_event (GtkWidget           *entry,
 {
   gboolean          expand;
   gboolean          is_expanded;
+  GtkTreePath      *path;
 
-  if (event->keyval == GDK_KEY_Up
-      || event->keyval == GDK_KEY_Down)
+  if (event->keyval == GDK_KEY_Tab &&
+      !gtk_widget_get_visible (window->paned) &&
+      xfce_appfinder_window_pointer_is_grabbed (entry))
+    {
+      /* don't tab to the close button */
+      return TRUE;
+    }
+
+  if (event->keyval == GDK_KEY_Up ||
+      event->keyval == GDK_KEY_Down)
     {
       expand = (event->keyval == GDK_KEY_Down);
       is_expanded = gtk_widget_get_visible (window->paned);
+
       if (is_expanded != expand)
         {
           /* don't break entry completion navigation in collapsed mode */
-          if (!is_expanded
-              && xfce_appfinder_window_pointer_is_grabbed (entry))
+          if (!is_expanded && xfce_appfinder_window_pointer_is_grabbed (entry))
             {
               /* window is still collapsed and the pointer is grabbed
                * by the popup menu, do nothing with the event */
@@ -1299,13 +1373,17 @@ xfce_appfinder_window_entry_key_press_event (GtkWidget           *entry,
           xfce_appfinder_window_set_expanded (window, expand);
           return TRUE;
         }
-    }
-  else if (event->keyval == GDK_KEY_Tab
-           && !gtk_widget_get_visible (window->paned)
-           && xfce_appfinder_window_pointer_is_grabbed (entry))
-    {
-      /* don't tab to the close button */
-      return TRUE;
+
+      /* The first item is usually selected, so we should jump to 2nd one
+       * when down key is pressed */
+      if (is_expanded && expand &&
+          GTK_IS_TREE_VIEW (window->view) && /* does not make sense for icon view */
+          xfce_appfinder_window_view_get_selected_path (window, &path))
+        {
+          gtk_tree_path_next (path);
+          gtk_tree_view_set_cursor (GTK_TREE_VIEW (window->view), path, NULL, FALSE);
+          gtk_tree_path_free (path);
+        }
     }
 
   return FALSE;
@@ -1367,15 +1445,49 @@ xfce_appfinder_window_treeview_key_press_event (GtkWidget           *widget,
                                                 GdkEventKey         *event,
                                                 XfceAppfinderWindow *window)
 {
+  GdkEvent     ev;
+  GtkTreePath *path;
+
   if (widget == window->view)
     {
+      if (event->keyval == GDK_KEY_Control_L ||
+          event->keyval == GDK_KEY_Control_R ||
+          event->keyval == GDK_KEY_Shift_L ||
+          event->keyval == GDK_KEY_Shift_R ||
+          event->keyval == GDK_KEY_Alt_L ||
+          event->keyval == GDK_KEY_Alt_R ||
+          event->keyval == GDK_KEY_Right ||
+          event->keyval == GDK_KEY_Down)
+          return FALSE;
+
       if (event->keyval == GDK_KEY_Left)
         {
-          gtk_widget_grab_focus (window->sidepane);
+          if (gtk_widget_get_realized (window->sidepane))
+            gtk_widget_grab_focus (window->sidepane);
           return TRUE;
         }
+
+      if (event->keyval == GDK_KEY_Up)
+        {
+          if (xfce_appfinder_window_view_get_selected_path (window, &path))
+            {
+              if (!gtk_tree_path_prev (path))
+                gtk_widget_grab_focus (window->entry);
+              gtk_tree_path_free (path);
+            }
+
+          return FALSE;
+        }
+
+      gtk_widget_grab_focus (window->entry);
+
+      ev.key = *event;
+      gtk_propagate_event (window->entry, &ev);
+
+      return TRUE;
     }
-  else if (widget == window->sidepane)
+
+  if (widget == window->sidepane)
     {
       if (event->keyval == GDK_KEY_Right)
         {
@@ -1424,7 +1536,7 @@ xfce_appfinder_window_category_changed (GtkTreeSelection    *selection,
           if (category == NULL)
             window->filter_category = NULL;
           else
-            window->filter_category = g_object_ref (G_OBJECT (category));
+            window->filter_category = GARCON_MENU_DIRECTORY (g_object_ref (G_OBJECT (category)));
 
           APPFINDER_DEBUG ("refilter category");
 
@@ -1433,7 +1545,7 @@ xfce_appfinder_window_category_changed (GtkTreeSelection    *selection,
             model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->view));
           else
             model = gtk_icon_view_get_model (GTK_ICON_VIEW (window->view));
-          gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (model));
+          gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (window->filter_model));
 
           /* store last category */
           if (xfconf_channel_get_bool (window->channel, "/remember-category", FALSE))
@@ -1671,8 +1783,8 @@ static void
 xfce_appfinder_window_execute (XfceAppfinderWindow *window,
                                gboolean             close_on_succeed)
 {
-  GtkTreeModel *model;
-  GtkTreeIter   iter, orig;
+  GtkTreeModel *model, *child_model;
+  GtkTreeIter   iter, child_iter;
   GError       *error = NULL;
   gboolean      result = FALSE;
   GdkScreen    *screen;
@@ -1693,12 +1805,21 @@ xfce_appfinder_window_execute (XfceAppfinderWindow *window,
 
       if (xfce_appfinder_window_view_get_selected (window, &model, &iter))
         {
-          gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &orig, &iter);
-          result = xfce_appfinder_model_execute (window->model, &orig, screen, &regular_command, &error);
+          child_model = model;
+
+          if (GTK_IS_TREE_MODEL_SORT (model) || xfce_appfinder_should_sort_icon_view ())
+            {
+              gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &child_iter, &iter);
+              iter = child_iter;
+              child_model = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+            }
+
+          gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (child_model), &child_iter, &iter);
+          result = xfce_appfinder_model_execute (window->model, &child_iter, screen, &regular_command, &error);
 
           if (!result && regular_command)
             {
-              gtk_tree_model_get (model, &iter, XFCE_APPFINDER_MODEL_COLUMN_COMMAND, &cmd, -1);
+              gtk_tree_model_get (model, &child_iter, XFCE_APPFINDER_MODEL_COLUMN_COMMAND, &cmd, -1);
               result = xfce_appfinder_window_execute_command (cmd, screen, window, FALSE, NULL, &error);
               g_free (cmd);
             }
@@ -1805,3 +1926,64 @@ xfce_appfinder_window_set_expanded (XfceAppfinderWindow *window,
   xfce_appfinder_window_entry_changed (window);
   xfce_appfinder_window_item_changed (window);
 }
+
+
+
+static gint
+xfce_appfinder_window_sort_items (GtkTreeModel *model,
+                                  GtkTreeIter  *a,
+                                  GtkTreeIter  *b,
+                                  gpointer      data)
+{
+  gchar        *normalized, *casefold, *title_a, *title_b, *found;
+  GtkWidget    *entry = GTK_WIDGET (data);
+  gint          result = -1;
+
+  appfinder_return_val_if_fail (GTK_IS_ENTRY (entry), 0);
+
+  normalized = g_utf8_normalize (gtk_entry_get_text (GTK_ENTRY (entry)), -1, G_NORMALIZE_ALL);
+  casefold = g_utf8_casefold (normalized, -1);
+  g_free (normalized);
+
+  gtk_tree_model_get (model, a, XFCE_APPFINDER_MODEL_COLUMN_TITLE, &title_a, -1);
+  normalized = g_utf8_normalize (title_a, -1, G_NORMALIZE_ALL);
+  title_a = g_utf8_casefold (normalized, -1);
+  g_free (normalized);
+
+  gtk_tree_model_get (model, b, XFCE_APPFINDER_MODEL_COLUMN_TITLE, &title_b, -1);
+  normalized = g_utf8_normalize (title_b, -1, G_NORMALIZE_ALL);
+  title_b = g_utf8_casefold (normalized, -1);
+  g_free (normalized);
+
+  if (g_strcmp0 (casefold, "") == 0)
+    result = g_strcmp0 (title_a, title_b);
+  else
+    {
+      found = g_strrstr (title_a, casefold);
+      if (found)
+        result -= (G_MAXINT - (found - title_a));
+
+      found = g_strrstr (title_b, casefold);
+      if (found)
+        result += (G_MAXINT - (found - title_b));
+    }
+
+  g_free (casefold);
+  g_free (title_a);
+  g_free (title_b);
+  return result;
+}
+
+
+
+/*
+ * Checks for gtk => 3.22.27 during runtime.
+ * This is necessary because sort model for icon view did not work as expected.
+ */
+static gboolean
+xfce_appfinder_should_sort_icon_view (void)
+{
+  return gtk_get_major_version () >= 3  &&
+         gtk_get_micro_version () >= 22 &&
+         gtk_get_micro_version () >= 27;
+}
